@@ -35,12 +35,12 @@ inline Type min(Type A, Type B) {
   return (B < A) ? B : A;
 }
 
-static void clearCache(IORequest &req, int blockSize) {
+static void clearCache(IORequest &req, unsigned int blockSize) {
   memset(req.data, 0, blockSize);
   req.dataLen = 0;
 }
 
-BlockFileIO::BlockFileIO(int blockSize, const FSConfigPtr &cfg)
+BlockFileIO::BlockFileIO(unsigned int blockSize, const FSConfigPtr &cfg)
     : _blockSize(blockSize), _allowHoles(cfg->config->allowHoles) {
   CHECK(_blockSize > 1);
   _cache.data = new unsigned char[_blockSize];
@@ -67,42 +67,55 @@ ssize_t BlockFileIO::cacheReadOneBlock(const IORequest &req) const {
    * in the last block of a file, which may be smaller than the blocksize.
    * For reverse encryption, the cache must not be used at all, because
    * the lower file may have changed behind our back. */
-  if ((_noCache == false) && (req.offset == _cache.offset) &&
-      (_cache.dataLen != 0)) {
+  if ((!_noCache) && (req.offset == _cache.offset) && (_cache.dataLen != 0)) {
     // satisfy request from cache
-    int len = req.dataLen;
-    if (_cache.dataLen < len) len = _cache.dataLen;  // Don't read past EOF
+    size_t len = req.dataLen;
+    if (_cache.dataLen < len) {
+      len = _cache.dataLen;  // Don't read past EOF
+    }
     memcpy(req.data, _cache.data, len);
     return len;
-  } else {
-    if (_cache.dataLen > 0) clearCache(_cache, _blockSize);
-
-    // cache results of read -- issue reads for full blocks
-    IORequest tmp;
-    tmp.offset = req.offset;
-    tmp.data = _cache.data;
-    tmp.dataLen = _blockSize;
-    ssize_t result = readOneBlock(tmp);
-    if (result > 0) {
-      _cache.offset = req.offset;
-      _cache.dataLen = result;  // the amount we really have
-      if (result > req.dataLen)
-        result = req.dataLen;  // only as much as requested
-      memcpy(req.data, _cache.data, result);
-    }
-    return result;
   }
+  if (_cache.dataLen > 0) {
+    clearCache(_cache, _blockSize);
+  }
+
+  // cache results of read -- issue reads for full blocks
+  IORequest tmp;
+  tmp.offset = req.offset;
+  tmp.data = _cache.data;
+  tmp.dataLen = _blockSize;
+  ssize_t result = readOneBlock(tmp);
+  if (result > 0) {
+    _cache.offset = req.offset;
+    _cache.dataLen = result;  // the amount we really have
+    if ((size_t)result > req.dataLen) {
+      result = req.dataLen;  // only as much as requested
+    }
+    memcpy(req.data, _cache.data, result);
+  }
+  return result;
 }
 
-bool BlockFileIO::cacheWriteOneBlock(const IORequest &req) {
-  // cache results of write (before pass-thru, because it may be modified
-  // in-place)
+ssize_t BlockFileIO::cacheWriteOneBlock(const IORequest &req) {
+  // Let's point request buffer to our own buffer, as it may be modified by
+  // encryption : originating process may not like to have its buffer modified
   memcpy(_cache.data, req.data, req.dataLen);
-  _cache.offset = req.offset;
-  _cache.dataLen = req.dataLen;
-  bool ok = writeOneBlock(req);
-  if (!ok) clearCache(_cache, _blockSize);
-  return ok;
+  IORequest tmp;
+  tmp.offset = req.offset;
+  tmp.data = _cache.data;
+  tmp.dataLen = req.dataLen;
+  ssize_t res = writeOneBlock(tmp);
+  if (res < 0) {
+    clearCache(_cache, _blockSize);
+  }
+  else {
+    // And now we can cache the write buffer from the request
+    memcpy(_cache.data, req.data, req.dataLen);
+    _cache.offset = req.offset;
+    _cache.dataLen = req.dataLen;
+  }
+  return res;
 }
 
 /**
@@ -111,11 +124,13 @@ bool BlockFileIO::cacheWriteOneBlock(const IORequest &req) {
  * data from the front of the first block if the request is not aligned.
  * Always requests aligned data of the size of one block or less from the
  * lower layer.
+ * Returns the number of bytes read, or -errno in case of failure.
  */
 ssize_t BlockFileIO::read(const IORequest &req) const {
   CHECK(_blockSize != 0);
 
-  int partialOffset = req.offset % _blockSize;
+  int partialOffset =
+      req.offset % _blockSize;  // can be int as _blockSize is int
   off_t blockNum = req.offset / _blockSize;
   ssize_t result = 0;
 
@@ -123,136 +138,170 @@ ssize_t BlockFileIO::read(const IORequest &req) const {
     // read completely within a single block -- can be handled as-is by
     // readOneBlock().
     return cacheReadOneBlock(req);
-  } else {
-    size_t size = req.dataLen;
+  }
+  size_t size = req.dataLen;
 
-    // if the request is larger then a block, then request each block
-    // individually
-    MemBlock mb;         // in case we need to allocate a temporary block..
-    IORequest blockReq;  // for requests we may need to make
-    blockReq.dataLen = _blockSize;
-    blockReq.data = NULL;
+  // if the request is larger then a block, then request each block
+  // individually
+  MemBlock mb;         // in case we need to allocate a temporary block..
+  IORequest blockReq;  // for requests we may need to make
+  blockReq.dataLen = _blockSize;
+  blockReq.data = nullptr;
 
-    unsigned char *out = req.data;
-    while (size) {
-      blockReq.offset = blockNum * _blockSize;
+  unsigned char *out = req.data;
+  while (size != 0u) {
+    blockReq.offset = blockNum * _blockSize;
 
-      // if we're reading a full block, then read directly into the
-      // result buffer instead of using a temporary
-      if (partialOffset == 0 && size >= (size_t)_blockSize)
-        blockReq.data = out;
-      else {
-        if (!mb.data) mb = MemoryPool::allocate(_blockSize);
-        blockReq.data = mb.data;
+    // if we're reading a full block, then read directly into the
+    // result buffer instead of using a temporary
+    if (partialOffset == 0 && size >= _blockSize) {
+      blockReq.data = out;
+    } else {
+      if (mb.data == nullptr) {
+        mb = MemoryPool::allocate(_blockSize);
       }
-
-      ssize_t readSize = cacheReadOneBlock(blockReq);
-      if (readSize <= partialOffset) break;  // didn't get enough bytes
-
-      int cpySize = min((size_t)(readSize - partialOffset), size);
-      CHECK(cpySize <= readSize);
-
-      // if we read to a temporary buffer, then move the data
-      if (blockReq.data != out)
-        memcpy(out, blockReq.data + partialOffset, cpySize);
-
-      result += cpySize;
-      size -= cpySize;
-      out += cpySize;
-      ++blockNum;
-      partialOffset = 0;
-
-      if (readSize < _blockSize) break;
+      blockReq.data = mb.data;
     }
 
-    if (mb.data) MemoryPool::release(mb);
+    ssize_t readSize = cacheReadOneBlock(blockReq);
+    if (readSize < 0) {
+      result = readSize;
+      break;
+    }
+    if (readSize <= partialOffset) {
+      break;  // didn't get enough bytes
+    }
+
+    size_t cpySize = min((size_t)readSize - (size_t)partialOffset, size);
+    CHECK(cpySize <= (size_t)readSize);
+
+    // if we read to a temporary buffer, then move the data
+    if (blockReq.data != out) {
+      memcpy(out, blockReq.data + partialOffset, cpySize);
+    }
+
+    result += cpySize;
+    size -= cpySize;
+    out += cpySize;
+    ++blockNum;
+    partialOffset = 0;
+
+    if ((size_t)readSize < _blockSize) {
+      break;
+    }
+  }
+
+  if (mb.data != nullptr) {
+    MemoryPool::release(mb);
   }
 
   return result;
 }
 
-bool BlockFileIO::write(const IORequest &req) {
+/**
+ * Returns the number of bytes written, or -errno in case of failure.
+ */
+ssize_t BlockFileIO::write(const IORequest &req) {
   CHECK(_blockSize != 0);
 
   off_t fileSize = getSize();
-  if (fileSize < 0) return false;
+  if (fileSize < 0) {
+    return fileSize;
+  }
 
   // where write request begins
   off_t blockNum = req.offset / _blockSize;
-  int partialOffset = req.offset % _blockSize;
+  int partialOffset =
+      req.offset % _blockSize;  // can be int as _blockSize is int
 
   // last block of file (for testing write overlaps with file boundary)
   off_t lastFileBlock = fileSize / _blockSize;
-  ssize_t lastBlockSize = fileSize % _blockSize;
+  size_t lastBlockSize = fileSize % _blockSize;
 
   off_t lastNonEmptyBlock = lastFileBlock;
-  if (lastBlockSize == 0) --lastNonEmptyBlock;
+  if (lastBlockSize == 0) {
+    --lastNonEmptyBlock;
+  }
 
   if (req.offset > fileSize) {
     // extend file first to fill hole with 0's..
     const bool forceWrite = false;
-    padFile(fileSize, req.offset, forceWrite);
+    int res = padFile(fileSize, req.offset, forceWrite);
+    if (res < 0) {
+      return res;
+    }
   }
 
   // check against edge cases where we can just let the base class handle the
   // request as-is..
   if (partialOffset == 0 && req.dataLen <= _blockSize) {
     // if writing a full block.. pretty safe..
-    if (req.dataLen == _blockSize) return cacheWriteOneBlock(req);
+    if (req.dataLen == _blockSize) {
+      return cacheWriteOneBlock(req);
+    }
 
     // if writing a partial block, but at least as much as what is
     // already there..
-    if (blockNum == lastFileBlock && req.dataLen >= lastBlockSize)
+    if (blockNum == lastFileBlock && req.dataLen >= lastBlockSize) {
       return cacheWriteOneBlock(req);
+    }
   }
 
   // have to merge data with existing block(s)..
   MemBlock mb;
 
   IORequest blockReq;
-  blockReq.data = NULL;
+  blockReq.data = nullptr;
   blockReq.dataLen = _blockSize;
 
-  bool ok = true;
+  ssize_t res = 0;
   size_t size = req.dataLen;
   unsigned char *inPtr = req.data;
-  while (size) {
+  while (size != 0u) {
     blockReq.offset = blockNum * _blockSize;
-    int toCopy = min((size_t)(_blockSize - partialOffset), size);
+    size_t toCopy = min((size_t)_blockSize - (size_t)partialOffset, size);
 
     // if writing an entire block, or writing a partial block that requires
     // no merging with existing data..
     if ((toCopy == _blockSize) ||
-        (partialOffset == 0 && blockReq.offset + toCopy >= fileSize)) {
+        (partialOffset == 0 && blockReq.offset + (off_t)toCopy >= fileSize)) {
       // write directly from buffer
       blockReq.data = inPtr;
       blockReq.dataLen = toCopy;
     } else {
       // need a temporary buffer, since we have to either merge or pad
       // the data.
-      if (!mb.data) mb = MemoryPool::allocate(_blockSize);
+      if (mb.data == nullptr) {
+        mb = MemoryPool::allocate(_blockSize);
+      }
       memset(mb.data, 0, _blockSize);
       blockReq.data = mb.data;
 
       if (blockNum > lastNonEmptyBlock) {
         // just pad..
-        blockReq.dataLen = toCopy + partialOffset;
+        blockReq.dataLen = partialOffset + toCopy;
       } else {
         // have to merge with existing block data..
         blockReq.dataLen = _blockSize;
-        blockReq.dataLen = cacheReadOneBlock(blockReq);
+        ssize_t readSize = cacheReadOneBlock(blockReq);
+        if (readSize < 0) {
+          res = readSize;
+          break;
+        }
+        blockReq.dataLen = readSize;
 
         // extend data if necessary..
-        if (partialOffset + toCopy > blockReq.dataLen)
+        if (partialOffset + toCopy > blockReq.dataLen) {
           blockReq.dataLen = partialOffset + toCopy;
+        }
       }
       // merge in the data to be written..
       memcpy(blockReq.data + partialOffset, inPtr, toCopy);
     }
 
     // Finally, write the damn thing!
-    if (!cacheWriteOneBlock(blockReq)) {
-      ok = false;
+    res = cacheWriteOneBlock(blockReq);
+    if (res < 0) {
       break;
     }
 
@@ -263,17 +312,26 @@ bool BlockFileIO::write(const IORequest &req) {
     partialOffset = 0;
   }
 
-  if (mb.data) MemoryPool::release(mb);
+  if (mb.data != nullptr) {
+    MemoryPool::release(mb);
+  }
 
-  return ok;
+  if (res < 0) {
+    return res;
+  }
+  return req.dataLen;
 }
 
-int BlockFileIO::blockSize() const { return _blockSize; }
+unsigned int BlockFileIO::blockSize() const { return _blockSize; }
 
-void BlockFileIO::padFile(off_t oldSize, off_t newSize, bool forceWrite) {
+/**
+ * Returns 0 in case of success, or -errno in case of failure.
+ */
+int BlockFileIO::padFile(off_t oldSize, off_t newSize, bool forceWrite) {
   off_t oldLastBlock = oldSize / _blockSize;
   off_t newLastBlock = newSize / _blockSize;
-  int newBlockSize = newSize % _blockSize;
+  int newBlockSize = newSize % _blockSize;  // can be int as _blockSize is int
+  ssize_t res = 0;
 
   IORequest req;
   MemBlock mb;
@@ -290,11 +348,12 @@ void BlockFileIO::padFile(off_t oldSize, off_t newSize, bool forceWrite) {
       req.dataLen = oldSize % _blockSize;
       int outSize = newSize % _blockSize;  // outSize > req.dataLen
 
-      if (outSize) {
+      if (outSize != 0) {
         memset(mb.data, 0, outSize);
-        cacheReadOneBlock(req);
-        req.dataLen = outSize;
-        cacheWriteOneBlock(req);
+        if ((res = cacheReadOneBlock(req)) >= 0) {
+          req.dataLen = outSize;
+          res = cacheWriteOneBlock(req);
+        }
       }
     } else
       VLOG(1) << "optimization: not padding last block";
@@ -313,37 +372,48 @@ void BlockFileIO::padFile(off_t oldSize, off_t newSize, bool forceWrite) {
     if (req.dataLen != 0) {
       VLOG(1) << "padding block " << oldLastBlock;
       memset(mb.data, 0, _blockSize);
-      cacheReadOneBlock(req);
-      req.dataLen = _blockSize;  // expand to full block size
-      cacheWriteOneBlock(req);
+      if ((res = cacheReadOneBlock(req)) >= 0) {
+        req.dataLen = _blockSize;  // expand to full block size
+        res = cacheWriteOneBlock(req);
+      }
       ++oldLastBlock;
     }
 
     // 2, pad zero blocks unless holes are allowed
     if (!_allowHoles) {
-      for (; oldLastBlock != newLastBlock; ++oldLastBlock) {
+      for (; (res >= 0) && (oldLastBlock != newLastBlock); ++oldLastBlock) {
         VLOG(1) << "padding block " << oldLastBlock;
         req.offset = oldLastBlock * _blockSize;
         req.dataLen = _blockSize;
         memset(mb.data, 0, req.dataLen);
-        cacheWriteOneBlock(req);
+        res = cacheWriteOneBlock(req);
       }
     }
 
     // 3. only necessary if write is forced and block is non 0 length
-    if (forceWrite && newBlockSize) {
+    if ((res >= 0) && forceWrite && (newBlockSize != 0)) {
       req.offset = newLastBlock * _blockSize;
       req.dataLen = newBlockSize;
       memset(mb.data, 0, req.dataLen);
-      cacheWriteOneBlock(req);
+      res = cacheWriteOneBlock(req);
     }
   }
 
-  if (mb.data) MemoryPool::release(mb);
+  if (mb.data != nullptr) {
+    MemoryPool::release(mb);
+  }
+
+  if (res < 0) {
+    return res;
+  }
+  return 0;
 }
 
+/**
+ * Returns 0 in case of success, or -errno in case of failure.
+ */
 int BlockFileIO::truncateBase(off_t size, FileIO *base) {
-  int partialBlock = size % _blockSize;
+  int partialBlock = size % _blockSize;  // can be int as _blockSize is int
   int res = 0;
 
   off_t oldSize = getSize();
@@ -353,13 +423,17 @@ int BlockFileIO::truncateBase(off_t size, FileIO *base) {
     // states that it will pad with 0's.
     // do the truncate so that the underlying filesystem can allocate
     // the space, and then we'll fill it in padFile..
-    if (base) base->truncate(size);
+    if (base != nullptr) {
+      res = base->truncate(size);
+    }
 
     const bool forceWrite = true;
-    padFile(oldSize, size, forceWrite);
+    if (res == 0) {
+      res = padFile(oldSize, size, forceWrite);
+    }
   } else if (size == oldSize) {
     // the easiest case, but least likely....
-  } else if (partialBlock) {
+  } else if (partialBlock != 0) {
     // partial block after truncate.  Need to read in the block being
     // truncated before the truncate.  Then write it back out afterwards,
     // since the encoding will change..
@@ -371,26 +445,32 @@ int BlockFileIO::truncateBase(off_t size, FileIO *base) {
     req.dataLen = _blockSize;
     req.data = mb.data;
 
-    ssize_t rdSz = cacheReadOneBlock(req);
+    ssize_t readSize = cacheReadOneBlock(req);
+    if (readSize < 0) {
+      res = readSize;
+    }
 
-    // do the truncate
-    if (base) res = base->truncate(size);
+    else if (base != nullptr) {
+      // do the truncate
+      res = base->truncate(size);
+    }
 
     // write back out partial block
     req.dataLen = partialBlock;
-    bool wrRes = cacheWriteOneBlock(req);
-
-    if ((rdSz < 0) || (!wrRes)) {
-      // rwarning - unlikely to ever occur..
-      RLOG(WARNING) << "truncate failure: read " << rdSz
-                    << " bytes, partial block of " << partialBlock;
+    if (res == 0) {
+      ssize_t writeSize = cacheWriteOneBlock(req);
+      if (writeSize < 0) {
+        res = writeSize;
+      }
     }
 
     MemoryPool::release(mb);
   } else {
     // truncating on a block bounday.  No need to re-encode the last
     // block..
-    if (base) res = base->truncate(size);
+    if (base != nullptr) {
+      res = base->truncate(size);
+    }
   }
 
   return res;
