@@ -38,28 +38,28 @@ namespace encfs {
 */
 class BlockFileIO : public FileIO {
  public:
-  BlockFileIO(int blockSize, const FSConfigPtr &cfg);
+  BlockFileIO(unsigned int blockSize, const FSConfigPtr &cfg);
   virtual ~BlockFileIO();
 
   // implemented in terms of blocks.
   virtual ssize_t read(const IORequest &req) const;
-  virtual bool write(const IORequest &req);
+  virtual ssize_t write(const IORequest &req);
 
-  virtual int blockSize() const;
+  virtual unsigned int blockSize() const;
 
  protected:
   int truncateBase(off_t size, FileIO *base);
-  void padFile(off_t oldSize, off_t newSize, bool forceWrite);
+  int padFile(off_t oldSize, off_t newSize, bool forceWrite);
 
   // same as read(), except that the request.offset field is guarenteed to be
   // block aligned, and the request size will not be larger then 1 block.
   virtual ssize_t readOneBlock(const IORequest &req) const = 0;
-  virtual bool writeOneBlock(const IORequest &req) = 0;
+  virtual ssize_t writeOneBlock(const IORequest &req) = 0;
 
   ssize_t cacheReadOneBlock(const IORequest &req) const;
-  bool cacheWriteOneBlock(const IORequest &req);
+  ssize_t cacheWriteOneBlock(const IORequest &req);
 
-  int _blockSize;
+  unsigned int _blockSize;
   bool _allowHoles;
   bool _noCache;
 
