@@ -3,9 +3,16 @@
 
 // Provides compatibility with RLog's rAssert, which throws an Error exception.
 
-#include "internal/easylogging++.h"
+#include "easylogging++.h"
 #include <stdexcept>
 
+// Cygwin / WinFsp does not support EBADMSG yet
+// https://github.com/billziss-gh/winfsp/issues/156
+#ifdef __CYGWIN__
+#undef EBADMSG
+#define EBADMSG EINVAL
+#endif
+
 namespace encfs {
 
 class Error : public std::runtime_error {
@@ -21,7 +28,7 @@ class Error : public std::runtime_error {
       RLOG(ERROR) << "Assert failed: " << STR(cond); \
       throw encfs::Error(STR(cond));                 \
     }                                                \
-  } while (0)
+  } while (false)
 
 void initLogging(bool enable_debug = false, bool is_daemon = false);
 
