@@ -67,6 +67,8 @@ struct EncFSConfig {
   int kdfIterations;
   long desiredKDFDuration;
 
+  bool plainData;         // do not encrypt file content
+
   int blockMACBytes;      // MAC headers on blocks..
   int blockMACRandBytes;  // number of random bytes in the block header
 
@@ -79,6 +81,7 @@ struct EncFSConfig {
   EncFSConfig() : keyData(), salt() {
     cfgType = Config_None;
     subVersion = 0;
+    plainData = false;
     blockMACBytes = 0;
     blockMACRandBytes = 0;
     uniqueIV = false;
@@ -130,7 +133,7 @@ struct FSConfig {
       : forceDecode(false), reverseEncryption(false), idleTracking(false) {}
 };
 
-typedef std::shared_ptr<FSConfig> FSConfigPtr;
+using FSConfigPtr = std::shared_ptr<FSConfig>;
 
 }  // namespace encfs
 
