@@ -22,11 +22,11 @@
 
 namespace encfs {
 
-FileIO::FileIO() {}
+FileIO::FileIO() = default;
 
-FileIO::~FileIO() {}
+FileIO::~FileIO() = default;
 
-int FileIO::blockSize() const { return 1; }
+unsigned int FileIO::blockSize() const { return 1; }
 
 bool FileIO::setIV(uint64_t iv) {
   (void)iv;
