@@ -60,7 +60,7 @@ struct EncFS_Root {
   ~EncFS_Root();
 };
 
-typedef std::shared_ptr<EncFS_Root> RootPtr;
+using RootPtr = std::shared_ptr<EncFS_Root>;
 
 enum ConfigMode { Config_Prompt, Config_Standard, Config_Paranoia };
 
@@ -72,10 +72,13 @@ enum ConfigMode { Config_Prompt, Config_Standard, Config_Paranoia };
 struct EncFS_Opts {
   std::string rootDir;
   std::string mountPoint;  // where to make filesystem visible
+  std::string unmountPoint;// same as mountPoint, but as given by the user
+  std::string cygDrive;    // Cygwin mount drive
   bool createIfNotFound;   // create filesystem if not found
   bool idleTracking;       // turn on idle monitoring of filesystem
   bool mountOnDemand;      // mounting on-demand
   bool delayMount;         // delay initial mount
+  bool unmount;            // unmount
 
   bool checkKey;     // check crypto key decoding
   bool forceDecode;  // force decode on MAC block failures
@@ -95,15 +98,19 @@ struct EncFS_Opts {
 
   bool readOnly;  // Mount read-only
 
+  bool insecure; // Allow to use plain data / to disable data encoding
+
   bool requireMac;  // Throw an error if MAC is disabled
 
   ConfigMode configMode;
+  std::string config;  // path to configuration file (or empty)
 
   EncFS_Opts() {
     createIfNotFound = true;
     idleTracking = false;
     mountOnDemand = false;
     delayMount = false;
+    unmount = false;
     checkKey = true;
     forceDecode = false;
     useStdin = false;
@@ -113,6 +120,7 @@ struct EncFS_Opts {
     configMode = Config_Prompt;
     noCache = false;
     readOnly = false;
+    insecure = false;
     requireMac = false;
   }
 };
@@ -120,19 +128,21 @@ struct EncFS_Opts {
 /*
     Read existing config file.  Looks for any supported configuration version.
 */
-ConfigType readConfig(const std::string &rootDir, EncFSConfig *config);
+ConfigType readConfig(const std::string &rootDir, EncFSConfig *config, const std::string &cmdConfig);
 
 /*
     Save the configuration.  Saves back as the same configuration type as was
     read from.
 */
 bool saveConfig(ConfigType type, const std::string &rootdir,
-                const EncFSConfig *config);
+                const EncFSConfig *config, const std::string &cmdConfig);
 
 class EncFS_Context;
 
 RootPtr initFS(EncFS_Context *ctx, const std::shared_ptr<EncFS_Opts> &opts);
 
+void unmountFS(const char *mountPoint);
+
 RootPtr createV6Config(EncFS_Context *ctx,
                        const std::shared_ptr<EncFS_Opts> &opts);
 
