@@ -37,8 +37,7 @@ class Interface {
           are implemented.
   */
   Interface(const char *name, int Current, int Revision, int Age);
-  Interface(const std::string &name, int Current, int Revision, int Age);
-  Interface(const Interface &src);
+  Interface(std::string name, int Current, int Revision, int Age);
   Interface();
 
   // check if we implement the interface described by B.
