@@ -44,7 +44,7 @@ class MACFileIO : public BlockFileIO {
       result in a warning message from encfs -- the garbled data will still
       be made available..
   */
-  MACFileIO(const std::shared_ptr<FileIO> &base, const FSConfigPtr &cfg);
+  MACFileIO(std::shared_ptr<FileIO> base, const FSConfigPtr &cfg);
   MACFileIO();
   virtual ~MACFileIO();
 
@@ -64,7 +64,7 @@ class MACFileIO : public BlockFileIO {
 
  private:
   virtual ssize_t readOneBlock(const IORequest &req) const;
-  virtual bool writeOneBlock(const IORequest &req);
+  virtual ssize_t writeOneBlock(const IORequest &req);
 
   std::shared_ptr<FileIO> base;
   std::shared_ptr<Cipher> cipher;
