@@ -21,6 +21,8 @@
 #ifndef _MemoryPool_incl_
 #define _MemoryPool_incl_
 
+namespace encfs {
+
 struct MemBlock {
   unsigned char *data;
 
@@ -46,4 +48,6 @@ void release(const MemBlock &el);
 void destroyAll();
 }
 
+}  // namespace encfs
+
 #endif
