@@ -49,24 +49,40 @@ static bool NullCipher_registered = Cipher::Register(
 
 class NullKey : public AbstractCipherKey {
  public:
-  NullKey() {}
-  virtual ~NullKey() {}
+  NullKey() = default;
+
+  // destructor
+  ~NullKey() override = default;
+
+  NullKey(const NullKey &src) = delete; // copy constructor
+  NullKey(NullKey&& other) = delete; // move constructor
+  NullKey& operator=(const NullKey& other) = delete; // copy assignment
+  NullKey& operator=(NullKey&& other) = delete; // move assignment
 };
 
 class NullDestructor {
  public:
-  NullDestructor() {}
-  NullDestructor(const NullDestructor &) {}
-  ~NullDestructor() {}
+  NullDestructor() = default;
+
+  // destructor
+  ~NullDestructor() = default;
+
+  // copy contructor
+  NullDestructor(const NullDestructor &) = default;
+
+  // move constructor
+  NullDestructor(NullDestructor &&) = default;
+
+  NullDestructor &operator=(const NullDestructor &) = delete; // copy assignment
+  NullDestructor& operator=(NullDestructor&& other) = delete; // move assignment
 
-  NullDestructor &operator=(const NullDestructor &) { return *this; }
   void operator()(NullKey *&) {}
 };
 std::shared_ptr<AbstractCipherKey> gNullKey(new NullKey(), NullDestructor());
 
 NullCipher::NullCipher(const Interface &iface_) { this->iface = iface_; }
 
-NullCipher::~NullCipher() {}
+NullCipher::~NullCipher() = default;
 
 Interface NullCipher::interface() const { return iface; }
 
