@@ -18,16 +18,17 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifdef linux
+#ifdef __linux__
 #define _XOPEN_SOURCE 500  // pick up pread , pwrite
 #endif
-#include "internal/easylogging++.h"
+#include "easylogging++.h"
 #include <cerrno>
+#include <cinttypes>
 #include <cstring>
 #include <fcntl.h>
-#include <inttypes.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <utility>
 
 #include "Error.h"
 #include "FileIO.h"
@@ -53,8 +54,8 @@ inline void swap(int &x, int &y) {
 RawFileIO::RawFileIO()
     : knownSize(false), fileSize(0), fd(-1), oldfd(-1), canWrite(false) {}
 
-RawFileIO::RawFileIO(const std::string &fileName)
-    : name(fileName),
+RawFileIO::RawFileIO(std::string fileName)
+    : name(std::move(fileName)),
       knownSize(false),
       fileSize(0),
       fd(-1),
@@ -68,9 +69,13 @@ RawFileIO::~RawFileIO() {
   swap(_fd, fd);
   swap(_oldfd, oldfd);
 
-  if (_oldfd != -1) close(_oldfd);
+  if (_oldfd != -1) {
+    close(_oldfd);
+  }
 
-  if (_fd != -1) close(_fd);
+  if (_fd != -1) {
+    close(_fd);
+  }
 }
 
 Interface RawFileIO::interface() const { return RawFileIO_iface; }
@@ -85,6 +90,8 @@ Interface RawFileIO::interface() const { return RawFileIO_iface; }
     This works around the problem described in
    https://github.com/vgough/encfs/issues/181
     Without this, "umask 0777 ; echo foo > bar" fails.
+
+    Sets errno when -1 is returned.
 */
 static int open_readonly_workaround(const char *path, int flags) {
   int fd = -1;
@@ -92,13 +99,11 @@ static int open_readonly_workaround(const char *path, int flags) {
   memset(&stbuf, 0, sizeof(struct stat));
   if (lstat(path, &stbuf) != -1) {
     // make sure user has read/write permission..
-    chmod(path, stbuf.st_mode | 0600);
-    fd = ::open(path, flags);
-    chmod(path, stbuf.st_mode);
-  } else {
-    RLOG(INFO) << "can't stat file " << path;
+    if (chmod(path, stbuf.st_mode | 0600) != -1) {
+      fd = ::open(path, flags);
+      chmod(path, stbuf.st_mode);
+    }
   }
-
   return fd;
 }
 
@@ -112,51 +117,56 @@ static int open_readonly_workaround(const char *path, int flags) {
        it..
 */
 int RawFileIO::open(int flags) {
-  bool requestWrite = ((flags & O_RDWR) || (flags & O_WRONLY));
+  bool requestWrite = (((flags & O_RDWR) != 0) || ((flags & O_WRONLY) != 0));
   VLOG(1) << "open call, requestWrite = " << requestWrite;
 
-  int result = 0;
-
   // if we have a descriptor and it is writable, or we don't need writable..
   if ((fd >= 0) && (canWrite || !requestWrite)) {
     VLOG(1) << "using existing file descriptor";
-    result = fd;  // success
-  } else {
-    int finalFlags = requestWrite ? O_RDWR : O_RDONLY;
+    return fd;  // success
+  }
+
+  int finalFlags = requestWrite ? O_RDWR : O_RDONLY;
 
 #if defined(O_LARGEFILE)
-    if (flags & O_LARGEFILE) finalFlags |= O_LARGEFILE;
+  if ((flags & O_LARGEFILE) != 0) {
+    finalFlags |= O_LARGEFILE;
+  }
 #else
 #warning O_LARGEFILE not supported
 #endif
 
-    int newFd = ::open(name.c_str(), finalFlags);
-
-    VLOG(1) << "open file with flags " << finalFlags << ", result = " << newFd;
-
-    if ((newFd == -1) && (errno == EACCES)) {
-      VLOG(1) << "using readonly workaround for open";
-      newFd = open_readonly_workaround(name.c_str(), finalFlags);
-    }
-
-    if (newFd >= 0) {
-      if (oldfd >= 0) {
-        RLOG(ERROR) << "leaking FD?: oldfd = " << oldfd << ", fd = " << fd
-                    << ", newfd = " << newFd;
-      }
-
-      // the old fd might still be in use, so just keep it around for
-      // now.
-      canWrite = requestWrite;
-      oldfd = fd;
-      result = fd = newFd;
-    } else {
-      result = -errno;
-      RLOG(DEBUG) << "::open error: " << strerror(errno);
-    }
+  int eno = 0;
+  int newFd = ::open(name.c_str(), finalFlags);
+  if (newFd < 0) {
+    eno = errno;
   }
 
-  return result;
+  VLOG(1) << "open file with flags " << finalFlags << ", result = " << newFd;
+
+  if ((newFd == -1) && (eno == EACCES)) {
+    VLOG(1) << "using readonly workaround for open";
+    newFd = open_readonly_workaround(name.c_str(), finalFlags);
+    eno = errno;
+  }
+
+  if (newFd < 0) {
+    RLOG(DEBUG) << "::open error: " << strerror(eno);
+    return -eno;
+  }
+
+  if (oldfd >= 0) {
+    RLOG(ERROR) << "leaking FD?: oldfd = " << oldfd << ", fd = " << fd
+                << ", newfd = " << newFd;
+  }
+
+  // the old fd might still be in use, so just keep it around for
+  // now.
+  canWrite = requestWrite;
+  oldfd = fd;
+  fd = newFd;
+
+  return fd;
 }
 
 int RawFileIO::getAttr(struct stat *stbuf) const {
@@ -184,13 +194,12 @@ off_t RawFileIO::getSize() const {
       const_cast<RawFileIO *>(this)->fileSize = stbuf.st_size;
       const_cast<RawFileIO *>(this)->knownSize = true;
       return fileSize;
-    } else {
-      RLOG(ERROR) << "getSize on " << name << " failed: " << strerror(errno);
-      return -1;
     }
-  } else {
-    return fileSize;
+    int eno = errno;
+    RLOG(ERROR) << "getSize on " << name << " failed: " << strerror(eno);
+    return -eno;
   }
+  return fileSize;
 }
 
 ssize_t RawFileIO::read(const IORequest &req) const {
@@ -199,51 +208,67 @@ ssize_t RawFileIO::read(const IORequest &req) const {
   ssize_t readSize = pread(fd, req.data, req.dataLen, req.offset);
 
   if (readSize < 0) {
+    int eno = errno;
     RLOG(WARNING) << "read failed at offset " << req.offset << " for "
-                  << req.dataLen << " bytes: " << strerror(errno);
+                  << req.dataLen << " bytes: " << strerror(eno);
+    return -eno;
   }
 
   return readSize;
 }
 
-bool RawFileIO::write(const IORequest &req) {
+ssize_t RawFileIO::write(const IORequest &req) {
   rAssert(fd >= 0);
-  rAssert(true == canWrite);
+  rAssert(canWrite);
 
-  int retrys = 10;
+  // int retrys = 10;
   void *buf = req.data;
   ssize_t bytes = req.dataLen;
   off_t offset = req.offset;
 
-  while (bytes && retrys > 0) {
+  /*
+   * Let's write while pwrite() writes, to avoid writing only a part of the
+   * request,
+   * whereas it could have been fully written. This to avoid inconsistencies /
+   * corruption.
+   */
+  // while ((bytes != 0) && retrys > 0) {
+  while (bytes != 0) {
     ssize_t writeSize = ::pwrite(fd, buf, bytes, offset);
 
     if (writeSize < 0) {
+      int eno = errno;
       knownSize = false;
       RLOG(WARNING) << "write failed at offset " << offset << " for " << bytes
-                    << " bytes: " << strerror(errno);
-      return false;
+                    << " bytes: " << strerror(eno);
+      // pwrite is not expected to return 0, so eno should always be set, but we
+      // never know...
+      return -eno;
+    }
+    if (writeSize == 0) {
+      return -EIO;
     }
 
     bytes -= writeSize;
     offset += writeSize;
     buf = (void *)((char *)buf + writeSize);
-    --retrys;
   }
 
-  if (bytes != 0) {
-    RLOG(ERROR) << "Write error: wrote " << req.dataLen - bytes << " bytes of "
-                << req.dataLen << ", max retries reached";
-    knownSize = false;
-    return false;
-  } else {
-    if (knownSize) {
-      off_t last = req.offset + req.dataLen;
-      if (last > fileSize) fileSize = last;
+  // if (bytes != 0) {
+  //   RLOG(ERROR) << "Write error: wrote " << req.dataLen - bytes << " bytes of
+  //   "
+  //               << req.dataLen << ", max retries reached";
+  //   knownSize = false;
+  //   return (eno) ? -eno : -EIO;
+  // }
+  if (knownSize) {
+    off_t last = req.offset + req.dataLen;
+    if (last > fileSize) {
+      fileSize = last;
     }
-
-    return true;
   }
+
+  return req.dataLen;
 }
 
 int RawFileIO::truncate(off_t size) {
@@ -251,11 +276,9 @@ int RawFileIO::truncate(off_t size) {
 
   if (fd >= 0 && canWrite) {
     res = ::ftruncate(fd, size);
-#if !defined(__FreeBSD__) && !defined(__APPLE__)
-    ::fdatasync(fd);
-#endif
-  } else
+  } else {
     res = ::truncate(name.c_str(), size);
+  }
 
   if (res < 0) {
     int eno = errno;
@@ -269,6 +292,14 @@ int RawFileIO::truncate(off_t size) {
     knownSize = true;
   }
 
+  if (fd >= 0 && canWrite) {
+#if defined(HAVE_FDATASYNC)
+    ::fdatasync(fd);
+#else
+    ::fsync(fd);
+#endif
+  }
+
   return res;
 }
 
