@@ -18,20 +18,21 @@
 #include "encfs.h"
 
 #include <cerrno>
+#include <cinttypes>
 #include <cstddef>
+#include <cstdint>
 #include <cstdio>
 #include <cstring>
+#include <ctime>
 #include <fcntl.h>
-#include <inttypes.h>
+#include <limits>
 #include <memory>
-#include <stdint.h>
 #include <sys/stat.h>
 #include <sys/statvfs.h>
 #include <sys/time.h>
-#include <time.h>
 #include <unistd.h>
 #include <utime.h>
-#ifdef linux
+#ifdef __linux__
 #include <sys/fsuid.h>
 #endif
 
@@ -41,7 +42,7 @@
 #include <attr/xattr.h>
 #endif
 
-#include "internal/easylogging++.h"
+#include "easylogging++.h"
 #include <functional>
 #include <string>
 #include <vector>
@@ -64,7 +65,7 @@ using namespace std::placeholders;
 
 namespace encfs {
 
-#define GET_FN(ctx, finfo) ctx->getNode((void *)(uintptr_t)finfo->fh)
+#define GET_FN(ctx, finfo) (ctx)->getNode((void *)(uintptr_t)(finfo)->fh)
 
 static EncFS_Context *context() {
   return (EncFS_Context *)fuse_get_context()->private_data;
@@ -75,21 +76,20 @@ static EncFS_Context *context() {
  * Optionally takes a pointer to the EncFS_Context, will get it from FUSE
  * if the argument is NULL.
  */
-static bool isReadOnly(EncFS_Context *ctx) {
-  if (ctx == NULL) ctx = (EncFS_Context *)fuse_get_context()->private_data;
-
-  return ctx->opts->readOnly;
-}
+static bool isReadOnly(EncFS_Context *ctx) { return ctx->opts->readOnly; }
 
 // helper function -- apply a functor to a cipher path, given the plain path
-static int withCipherPath(const char *opName, const char *path,
-                          function<int(EncFS_Context *, const string &)> op,
-                          bool passReturnCode = false) {
+static int withCipherPath(
+    const char *opName, const char *path,
+    const function<int(EncFS_Context *, const string &)> &op,
+    bool passReturnCode = false) {
   EncFS_Context *ctx = context();
 
   int res = -EIO;
   std::shared_ptr<DirNode> FSRoot = ctx->getRoot(&res);
-  if (!FSRoot) return res;
+  if (!FSRoot) {
+    return res;
+  }
 
   try {
     string cyName = FSRoot->cipherPath(path);
@@ -111,21 +111,23 @@ static int withCipherPath(const char *opName, const char *path,
   return res;
 }
 
-static void checkCanary(std::shared_ptr<FileNode> fnode) {
-  if(fnode->canary == CANARY_OK) {
+static void checkCanary(const std::shared_ptr<FileNode> &fnode) {
+  if (fnode->canary == CANARY_OK) {
     return;
   }
-  if(fnode->canary == CANARY_RELEASED) {
+  if (fnode->canary == CANARY_RELEASED) {
     // "fnode" may have been released after it was retrieved by
     // lookupFuseFh. This is not an error. std::shared_ptr will release
     // the memory only when all operations on the FileNode have been
     // completed.
     return;
   }
-  if(fnode->canary == CANARY_DESTROYED) {
-    RLOG(ERROR) << "canary=CANARY_DESTROYED. FileNode accessed after it was destroyed.";
+  if (fnode->canary == CANARY_DESTROYED) {
+    RLOG(ERROR)
+        << "canary=CANARY_DESTROYED. FileNode accessed after it was destroyed.";
   } else {
-    RLOG(ERROR) << "canary=0x" << std::hex << fnode->canary << ". Memory corruption?";
+    RLOG(ERROR) << "canary=0x" << std::hex << fnode->canary
+                << ". Memory corruption?";
   }
   throw Error("dead canary");
 }
@@ -133,12 +135,18 @@ static void checkCanary(std::shared_ptr<FileNode> fnode) {
 // helper function -- apply a functor to a node
 static int withFileNode(const char *opName, const char *path,
                         struct fuse_file_info *fi,
-                        function<int(FileNode *)> op) {
+                        const function<int(FileNode *)> &op) {
   EncFS_Context *ctx = context();
 
   int res = -EIO;
-  std::shared_ptr<DirNode> FSRoot = ctx->getRoot(&res);
-  if (!FSRoot) return res;
+  bool skipUsageCount = false;
+  if (strlen(path) == 1) {
+    skipUsageCount = true;
+  }
+  std::shared_ptr<DirNode> FSRoot = ctx->getRoot(&res, skipUsageCount);
+  if (!FSRoot) {
+    return res;
+  }
 
   try {
 
@@ -159,6 +167,13 @@ static int withFileNode(const char *opName, const char *path,
     if (fi != nullptr && fi->fh != 0) {
       auto node = ctx->lookupFuseFh(fi->fh);
       if (node == nullptr) {
+#ifdef __CYGWIN__
+        if (strcmp(opName, "flush") == 0) {
+          RLOG(WARNING) << "Filenode to flush not found, file has certainly be renamed: "
+                        << path;
+          return 0;
+        }
+#endif
         auto msg = "fh=" + std::to_string(fi->fh) + " not found in fuseFhMap";
         throw Error(msg.c_str());
       }
@@ -214,7 +229,7 @@ int _do_getattr(FileNode *fnode, struct stat *stbuf) {
 }
 
 int encfs_getattr(const char *path, struct stat *stbuf) {
-  return withFileNode("getattr", path, NULL, bind(_do_getattr, _1, stbuf));
+  return withFileNode("getattr", path, nullptr, bind(_do_getattr, _1, stbuf));
 }
 
 int encfs_fgetattr(const char *path, struct stat *stbuf,
@@ -226,9 +241,15 @@ int encfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
                   off_t offset, struct fuse_file_info *finfo) {
   EncFS_Context *ctx = context();
 
+  //unused parameters
+  (void)offset;
+  (void)finfo;
+
   int res = ESUCCESS;
   std::shared_ptr<DirNode> FSRoot = ctx->getRoot(&res);
-  if (!FSRoot) return res;
+  if (!FSRoot) {
+    return res;
+  }
 
   try {
 
@@ -250,7 +271,9 @@ int encfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
 #if defined(fuse_fill_dir_flags)
         if (filler(buf, name.c_str(), &st, 0, 0)) break;
 #else
-        if (filler(buf, name.c_str(), &st, 0)) break;
+        if (filler(buf, name.c_str(), &st, 0) != 0) {
+          break;
+        }
 #endif
 
         name = dt.nextPlaintextName(&fileType, &inode);
@@ -269,11 +292,15 @@ int encfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
 int encfs_mknod(const char *path, mode_t mode, dev_t rdev) {
   EncFS_Context *ctx = context();
 
-  if (isReadOnly(ctx)) return -EROFS;
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
 
   int res = -EIO;
   std::shared_ptr<DirNode> FSRoot = ctx->getRoot(&res);
-  if (!FSRoot) return res;
+  if (!FSRoot) {
+    return res;
+  }
 
   try {
     std::shared_ptr<FileNode> fnode = FSRoot->lookupNode(path, "mknod");
@@ -298,8 +325,9 @@ int encfs_mknod(const char *path, mode_t mode, dev_t rdev) {
           FSRoot->lookupNode(parent.c_str(), "mknod");
 
       struct stat st;
-      if (dnode->getAttr(&st) == 0)
+      if (dnode->getAttr(&st) == 0) {
         res = fnode->mknod(mode, rdev, uid, st.st_gid);
+      }
     }
   } catch (encfs::Error &err) {
     RLOG(ERROR) << "error caught in mknod: " << err.what();
@@ -311,11 +339,15 @@ int encfs_mkdir(const char *path, mode_t mode) {
   fuse_context *fctx = fuse_get_context();
   EncFS_Context *ctx = context();
 
-  if (isReadOnly(ctx)) return -EROFS;
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
 
   int res = -EIO;
   std::shared_ptr<DirNode> FSRoot = ctx->getRoot(&res);
-  if (!FSRoot) return res;
+  if (!FSRoot) {
+    return res;
+  }
 
   try {
     uid_t uid = 0;
@@ -333,8 +365,9 @@ int encfs_mkdir(const char *path, mode_t mode) {
           FSRoot->lookupNode(parent.c_str(), "mkdir");
 
       struct stat st;
-      if (dnode->getAttr(&st) == 0)
+      if (dnode->getAttr(&st) == 0) {
         res = FSRoot->mkdir(path, mode, uid, st.st_gid);
+      }
     }
   } catch (encfs::Error &err) {
     RLOG(ERROR) << "error caught in mkdir: " << err.what();
@@ -344,12 +377,15 @@ int encfs_mkdir(const char *path, mode_t mode) {
 
 int encfs_unlink(const char *path) {
   EncFS_Context *ctx = context();
-
-  if (isReadOnly(ctx)) return -EROFS;
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
 
   int res = -EIO;
   std::shared_ptr<DirNode> FSRoot = ctx->getRoot(&res);
-  if (!FSRoot) return res;
+  if (!FSRoot) {
+    return res;
+  }
 
   try {
     // let DirNode handle it atomically so that it can handle race
@@ -366,7 +402,10 @@ int _do_rmdir(EncFS_Context *, const string &cipherPath) {
 }
 
 int encfs_rmdir(const char *path) {
-  if (isReadOnly(NULL)) return -EROFS;
+  EncFS_Context *ctx = context();
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
   return withCipherPath("rmdir", path, bind(_do_rmdir, _1, _2));
 }
 
@@ -374,29 +413,28 @@ int _do_readlink(EncFS_Context *ctx, const string &cyName, char *buf,
                  size_t size) {
   int res = ESUCCESS;
   std::shared_ptr<DirNode> FSRoot = ctx->getRoot(&res);
-  if (!FSRoot) return res;
+  if (!FSRoot) {
+    return res;
+  }
 
   res = ::readlink(cyName.c_str(), buf, size - 1);
 
-  if (res == -1) return -errno;
+  if (res == -1) {
+    return -errno;
+  }
 
   buf[res] = '\0';  // ensure null termination
   string decodedName;
-  try {
-    decodedName = FSRoot->plainPath(buf);
-  } catch (...) {
-    VLOG(1) << "caught error decoding path";
-  }
+  decodedName = FSRoot->plainPath(buf);
 
   if (!decodedName.empty()) {
     strncpy(buf, decodedName.c_str(), size - 1);
     buf[size - 1] = '\0';
 
     return ESUCCESS;
-  } else {
-    RLOG(WARNING) << "Error decoding link";
-    return -1;
   }
+  RLOG(WARNING) << "Error decoding link";
+  return -1;
 }
 
 int encfs_readlink(const char *path, char *buf, size_t size) {
@@ -410,11 +448,15 @@ int encfs_readlink(const char *path, char *buf, size_t size) {
 int encfs_symlink(const char *to, const char *from) {
   EncFS_Context *ctx = context();
 
-  if (isReadOnly(ctx)) return -EROFS;
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
 
   int res = -EIO;
   std::shared_ptr<DirNode> FSRoot = ctx->getRoot(&res);
-  if (!FSRoot) return res;
+  if (!FSRoot) {
+    return res;
+  }
 
   try {
     string fromCName = FSRoot->cipherPath(from);
@@ -429,34 +471,61 @@ int encfs_symlink(const char *to, const char *from) {
     int oldgid = -1;
     if (ctx->publicFilesystem) {
       fuse_context *context = fuse_get_context();
-      olduid = setfsuid(context->uid);
       oldgid = setfsgid(context->gid);
+      if (oldgid == -1) {
+        int eno = errno;
+        RLOG(DEBUG) << "setfsgid error: " << strerror(eno);
+        return -EPERM;
+      }
+      olduid = setfsuid(context->uid);
+      if (olduid == -1) {
+        int eno = errno;
+        RLOG(DEBUG) << "setfsuid error: " << strerror(eno);
+        return -EPERM;
+      }
     }
     res = ::symlink(toCName.c_str(), fromCName.c_str());
-    if (olduid >= 0) setfsuid(olduid);
-    if (oldgid >= 0) setfsgid(oldgid);
+    if (olduid >= 0) {
+      if(setfsuid(olduid) == -1) {
+        int eno = errno;
+        RLOG(DEBUG) << "setfsuid back error: " << strerror(eno);
+        // does not return error here as initial setfsuid worked
+      }
+    }
+    if (oldgid >= 0) {
+      if(setfsgid(oldgid) == -1) {
+        int eno = errno;
+        RLOG(DEBUG) << "setfsgid back error: " << strerror(eno);
+        // does not return error here as initial setfsgid worked
+      }
+    }
 
-    if (res == -1)
+    if (res == -1) {
       res = -errno;
-    else
+    } else {
       res = ESUCCESS;
+    }
   } catch (encfs::Error &err) {
     RLOG(ERROR) << "error caught in symlink: " << err.what();
   }
   return res;
 }
 
-int encfs_link(const char *from, const char *to) {
+int encfs_link(const char *to, const char *from) {
   EncFS_Context *ctx = context();
 
-  if (isReadOnly(ctx)) return -EROFS;
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
 
   int res = -EIO;
   std::shared_ptr<DirNode> FSRoot = ctx->getRoot(&res);
-  if (!FSRoot) return res;
+  if (!FSRoot) {
+    return res;
+  }
 
   try {
-    res = FSRoot->link(from, to);
+    res = FSRoot->link(to, from);
   } catch (encfs::Error &err) {
     RLOG(ERROR) << "error caught in link: " << err.what();
   }
@@ -466,11 +535,15 @@ int encfs_link(const char *from, const char *to) {
 int encfs_rename(const char *from, const char *to) {
   EncFS_Context *ctx = context();
 
-  if (isReadOnly(ctx)) return -EROFS;
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
 
   int res = -EIO;
   std::shared_ptr<DirNode> FSRoot = ctx->getRoot(&res);
-  if (!FSRoot) return res;
+  if (!FSRoot) {
+    return res;
+  }
 
   try {
     res = FSRoot->rename(from, to);
@@ -485,7 +558,10 @@ int _do_chmod(EncFS_Context *, const string &cipherPath, mode_t mode) {
 }
 
 int encfs_chmod(const char *path, mode_t mode) {
-  if (isReadOnly(NULL)) return -EROFS;
+  EncFS_Context *ctx = context();
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
   return withCipherPath("chmod", path, bind(_do_chmod, _1, _2, mode));
 }
 
@@ -495,19 +571,28 @@ int _do_chown(EncFS_Context *, const string &cyName, uid_t u, gid_t g) {
 }
 
 int encfs_chown(const char *path, uid_t uid, gid_t gid) {
-  if (isReadOnly(NULL)) return -EROFS;
+  EncFS_Context *ctx = context();
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
   return withCipherPath("chown", path, bind(_do_chown, _1, _2, uid, gid));
 }
 
 int _do_truncate(FileNode *fnode, off_t size) { return fnode->truncate(size); }
 
 int encfs_truncate(const char *path, off_t size) {
-  if (isReadOnly(NULL)) return -EROFS;
-  return withFileNode("truncate", path, NULL, bind(_do_truncate, _1, size));
+  EncFS_Context *ctx = context();
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
+  return withFileNode("truncate", path, nullptr, bind(_do_truncate, _1, size));
 }
 
 int encfs_ftruncate(const char *path, off_t size, struct fuse_file_info *fi) {
-  if (isReadOnly(NULL)) return -EROFS;
+  EncFS_Context *ctx = context();
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
   return withFileNode("ftruncate", path, fi, bind(_do_truncate, _1, size));
 }
 
@@ -517,7 +602,10 @@ int _do_utime(EncFS_Context *, const string &cyName, struct utimbuf *buf) {
 }
 
 int encfs_utime(const char *path, struct utimbuf *buf) {
-  if (isReadOnly(NULL)) return -EROFS;
+  EncFS_Context *ctx = context();
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
   return withCipherPath("utime", path, bind(_do_utime, _1, _2, buf));
 }
 
@@ -538,19 +626,26 @@ int _do_utimens(EncFS_Context *, const string &cyName,
 }
 
 int encfs_utimens(const char *path, const struct timespec ts[2]) {
-  if (isReadOnly(NULL)) return -EROFS;
+  EncFS_Context *ctx = context();
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
   return withCipherPath("utimens", path, bind(_do_utimens, _1, _2, ts));
 }
 
 int encfs_open(const char *path, struct fuse_file_info *file) {
   EncFS_Context *ctx = context();
 
-  if (isReadOnly(ctx) && (file->flags & O_WRONLY || file->flags & O_RDWR))
+  if (isReadOnly(ctx) &&
+      (((file->flags & O_WRONLY) != 0) || ((file->flags & O_RDWR) != 0))) {
     return -EROFS;
+  }
 
   int res = -EIO;
   std::shared_ptr<DirNode> FSRoot = ctx->getRoot(&res);
-  if (!FSRoot) return res;
+  if (!FSRoot) {
+    return res;
+  }
 
   try {
     std::shared_ptr<FileNode> fnode =
@@ -575,7 +670,7 @@ int encfs_open(const char *path, struct fuse_file_info *file) {
 
 int encfs_create(const char *path, mode_t mode, struct fuse_file_info *file) {
   int res = encfs_mknod(path, mode, 0);
-  if (res) {
+  if (res != 0) {
     return res;
   }
 
@@ -626,12 +721,17 @@ int encfs_release(const char *path, struct fuse_file_info *finfo) {
   }
 }
 
-int _do_read(FileNode *fnode, unsigned char *ptr, size_t size, off_t off) {
+ssize_t _do_read(FileNode *fnode, unsigned char *ptr, size_t size, off_t off) {
   return fnode->read(off, ptr, size);
 }
 
 int encfs_read(const char *path, char *buf, size_t size, off_t offset,
                struct fuse_file_info *file) {
+  // Unfortunately we have to convert from ssize_t (pread) to int (fuse), so
+  // let's check this will be OK
+  if (size > std::numeric_limits<int>::max()) {
+    size = std::numeric_limits<int>::max();
+  }
   return withFileNode("read", path, file,
                       bind(_do_read, _1, (unsigned char *)buf, size, offset));
 }
@@ -641,20 +741,29 @@ int _do_fsync(FileNode *fnode, int dataSync) {
 }
 
 int encfs_fsync(const char *path, int dataSync, struct fuse_file_info *file) {
-  if (isReadOnly(NULL)) return -EROFS;
+  EncFS_Context *ctx = context();
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
   return withFileNode("fsync", path, file, bind(_do_fsync, _1, dataSync));
 }
 
-int _do_write(FileNode *fnode, unsigned char *ptr, size_t size, off_t offset) {
-  if (fnode->write(offset, ptr, size))
-    return size;
-  else
-    return -EIO;
+ssize_t _do_write(FileNode *fnode, unsigned char *ptr, size_t size,
+                  off_t offset) {
+  return fnode->write(offset, ptr, size);
 }
 
 int encfs_write(const char *path, const char *buf, size_t size, off_t offset,
                 struct fuse_file_info *file) {
-  if (isReadOnly(NULL)) return -EROFS;
+  // Unfortunately we have to convert from ssize_t (pwrite) to int (fuse), so
+  // let's check this will be OK
+  if (size > std::numeric_limits<int>::max()) {
+    size = std::numeric_limits<int>::max();
+  }
+  EncFS_Context *ctx = context();
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
   return withFileNode("write", path, file,
                       bind(_do_write, _1, (unsigned char *)buf, size, offset));
 }
@@ -666,16 +775,18 @@ int encfs_statfs(const char *path, struct statvfs *st) {
   int res = -EIO;
   try {
     (void)path;  // path should always be '/' for now..
-    rAssert(st != NULL);
+    rAssert(st != nullptr);
     string cyName = ctx->rootCipherDir;
 
     VLOG(1) << "doing statfs of " << cyName;
     res = statvfs(cyName.c_str(), st);
-    if (!res) {
+    if (res == 0) {
       // adjust maximum name length..
       st->f_namemax = 6 * (st->f_namemax - 2) / 8;  // approx..
     }
-    if (res == -1) res = -errno;
+    if (res == -1) {
+      res = -errno;
+    }
   } catch (encfs::Error &err) {
     RLOG(ERROR) << "error caught in statfs: " << err.what();
   }
@@ -687,12 +798,15 @@ int encfs_statfs(const char *path, struct statvfs *st) {
 #ifdef XATTR_ADD_OPT
 int _do_setxattr(EncFS_Context *, const string &cyName, const char *name,
                  const char *value, size_t size, uint32_t pos) {
-  int options = 0;
+  int options = XATTR_NOFOLLOW;
   return ::setxattr(cyName.c_str(), name, value, size, pos, options);
 }
 int encfs_setxattr(const char *path, const char *name, const char *value,
                    size_t size, int flags, uint32_t position) {
-  if (isReadOnly(NULL)) return -EROFS;
+  EncFS_Context *ctx = context();
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
   (void)flags;
   return withCipherPath("setxattr", path, bind(_do_setxattr, _1, _2, name,
                                                value, size, position));
@@ -700,11 +814,14 @@ int encfs_setxattr(const char *path, const char *name, const char *value,
 #else
 int _do_setxattr(EncFS_Context *, const string &cyName, const char *name,
                  const char *value, size_t size, int flags) {
-  return ::setxattr(cyName.c_str(), name, value, size, flags);
+  return ::lsetxattr(cyName.c_str(), name, value, size, flags);
 }
 int encfs_setxattr(const char *path, const char *name, const char *value,
                    size_t size, int flags) {
-  if (isReadOnly(NULL)) return -EROFS;
+  EncFS_Context *ctx = context();
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
   return withCipherPath("setxattr", path,
                         bind(_do_setxattr, _1, _2, name, value, size, flags));
 }
@@ -713,7 +830,7 @@ int encfs_setxattr(const char *path, const char *name, const char *value,
 #ifdef XATTR_ADD_OPT
 int _do_getxattr(EncFS_Context *, const string &cyName, const char *name,
                  void *value, size_t size, uint32_t pos) {
-  int options = 0;
+  int options = XATTR_NOFOLLOW;
   return ::getxattr(cyName.c_str(), name, value, size, pos, options);
 }
 int encfs_getxattr(const char *path, const char *name, char *value, size_t size,
@@ -725,7 +842,7 @@ int encfs_getxattr(const char *path, const char *name, char *value, size_t size,
 #else
 int _do_getxattr(EncFS_Context *, const string &cyName, const char *name,
                  void *value, size_t size) {
-  return ::getxattr(cyName.c_str(), name, value, size);
+  return ::lgetxattr(cyName.c_str(), name, value, size);
 }
 int encfs_getxattr(const char *path, const char *name, char *value,
                    size_t size) {
@@ -737,12 +854,9 @@ int encfs_getxattr(const char *path, const char *name, char *value,
 
 int _do_listxattr(EncFS_Context *, const string &cyName, char *list,
                   size_t size) {
-#ifndef XATTR_LLIST
-#define llistxattr listxattr
-#endif
 #ifdef XATTR_ADD_OPT
-  int options = 0;
-  int res = ::llistxattr(cyName.c_str(), list, size, options);
+  int options = XATTR_NOFOLLOW;
+  int res = ::listxattr(cyName.c_str(), list, size, options);
 #else
   int res = ::llistxattr(cyName.c_str(), list, size);
 #endif
@@ -756,16 +870,19 @@ int encfs_listxattr(const char *path, char *list, size_t size) {
 
 int _do_removexattr(EncFS_Context *, const string &cyName, const char *name) {
 #ifdef XATTR_ADD_OPT
-  int options = 0;
+  int options = XATTR_NOFOLLOW;
   int res = ::removexattr(cyName.c_str(), name, options);
 #else
-  int res = ::removexattr(cyName.c_str(), name);
+  int res = ::lremovexattr(cyName.c_str(), name);
 #endif
   return (res == -1) ? -errno : res;
 }
 
 int encfs_removexattr(const char *path, const char *name) {
-  if (isReadOnly(NULL)) return -EROFS;
+  EncFS_Context *ctx = context();
+  if (isReadOnly(ctx)) {
+    return -EROFS;
+  }
 
   return withCipherPath("removexattr", path,
                         bind(_do_removexattr, _1, _2, name));
