@@ -34,6 +34,7 @@
 
 #include "Cipher.h"
 #include "CipherKey.h"
+#include "Context.h"
 #include "DirNode.h"
 #include "Error.h"
 #include "FSConfig.h"
@@ -95,7 +96,7 @@ struct CommandOpts {
     {"showcruft", 1, 1, cmd_showcruft, "(root dir)",
      // xgroup(usage)
      gettext_noop("  -- show undecodable filenames in the volume")},
-    {"cat", 2, 3, cmd_cat, "[--extpass=prog] (root dir) path",
+    {"cat", 2, 4, cmd_cat, "[--extpass=prog] [--reverse] (root dir) path",
      // xgroup(usage)
      gettext_noop("  -- decodes the file and cats it to standard out")},
     {"decode", 1, 100, cmd_decode,
@@ -114,6 +115,8 @@ struct CommandOpts {
      gettext_noop("  -- print version number and exit")},
     {0, 0, 0, 0, 0, 0}};
 
+auto ctx = std::make_shared<EncFS_Context>();
+
 static void usage(const char *name) {
   cerr << autosprintf(_("encfsctl version %s"), VERSION) << "\n"
        << _("Usage:\n")
@@ -164,7 +167,7 @@ static int showInfo(int argc, char **argv) {
   if (!checkDir(rootDir)) return EXIT_FAILURE;
 
   std::shared_ptr<EncFSConfig> config(new EncFSConfig);
-  ConfigType type = readConfig(rootDir, config.get());
+  ConfigType type = readConfig(rootDir, config.get(), "");
 
   // show information stored in config..
   switch (type) {
@@ -219,7 +222,7 @@ static RootPtr initRootInfo(int &argc, char **&argv) {
   opts->createIfNotFound = false;
   opts->checkKey = false;
 
-  static struct option long_options[] = {{"extpass", 1, 0, 'p'}, {0, 0, 0, 0}};
+  static struct option long_options[] = {{"extpass", 1, 0, 'p'}, {"reverse", 0, nullptr, 'r'}, {0, 0, 0, 0}};
 
   for (;;) {
     int option_index = 0;
@@ -231,6 +234,9 @@ static RootPtr initRootInfo(int &argc, char **&argv) {
       case 'p':
         opts->passwordProgram.assign(optarg);
         break;
+      case 'r':
+        opts->reverseEncryption = true;
+        break;
       default:
         RLOG(WARNING) << "getopt error: " << res;
         break;
@@ -248,7 +254,8 @@ static RootPtr initRootInfo(int &argc, char **&argv) {
     --argc;
     ++argv;
 
-    if (checkDir(opts->rootDir)) result = initFS(NULL, opts);
+    ctx->publicFilesystem = opts->ownerCreate;
+    if (checkDir(opts->rootDir)) result = initFS(ctx.get(), opts);
 
     if (!result)
       cerr << _("Unable to initialize encrypted filesystem - check path.\n");
@@ -266,7 +273,9 @@ static RootPtr initRootInfo(const char *crootDir) {
     opts->rootDir = rootDir;
     opts->createIfNotFound = false;
     opts->checkKey = false;
-    result = initFS(NULL, opts);
+
+    ctx->publicFilesystem = opts->ownerCreate;
+    result = initFS(ctx.get(), opts);
   }
 
   if (!result)
@@ -367,13 +376,19 @@ template <typename T>
 int processContents(const std::shared_ptr<EncFS_Root> &rootInfo,
                     const char *path, T &op) {
   int errCode = 0;
-  std::shared_ptr<FileNode> node =
-      rootInfo->root->openNode(path, "encfsctl", O_RDONLY, &errCode);
+  std::shared_ptr<FileNode> node;
+
+  try {
+    node = rootInfo->root->openNode(path, "encfsctl", O_RDONLY, &errCode);
+  }
+  catch(...) {}
 
   if (!node) {
     // try treating filename as an enciphered path
     string plainName = rootInfo->root->plainPath(path);
-    node = rootInfo->root->lookupNode(plainName.c_str(), "encfsctl");
+    if (plainName.length() > 0) {
+      node = rootInfo->root->lookupNode(plainName.c_str(), "encfsctl");
+    }
     if (node) {
       errCode = node->open(O_RDONLY);
       if (errCode < 0) node.reset();
@@ -414,6 +429,11 @@ static int cmd_cat(int argc, char **argv) {
   if (!rootInfo) return EXIT_FAILURE;
 
   const char *path = argv[0];
+  // If user provides a leading slash, in reverse mode, it will be converted
+  // to "+" by plainpath, and will fail to decode... Workaround below then...
+  if (path[0] == '/') {
+    path++;
+  }
   WriteOutput output(STDOUT_FILENO);
   int errCode = processContents(rootInfo, path, output);
 
@@ -622,7 +642,7 @@ static int do_chpasswd(bool useStdin, bool annotate, bool checkOnly, int argc,
   if (!checkDir(rootDir)) return EXIT_FAILURE;
 
   EncFSConfig *config = new EncFSConfig;
-  ConfigType cfgType = readConfig(rootDir, config);
+  ConfigType cfgType = readConfig(rootDir, config, "");
 
   if (cfgType == Config_None) {
     cout << _("Unable to load or parse config file\n");
@@ -683,7 +703,7 @@ static int do_chpasswd(bool useStdin, bool annotate, bool checkOnly, int argc,
     config->assignKeyData(keyBuf, encodedKeySize);
     delete[] keyBuf;
 
-    if (saveConfig(cfgType, rootDir, config)) {
+    if (saveConfig(cfgType, rootDir, config, "")) {
       // password modified -- changes volume key of filesystem..
       cout << _("Volume Key successfully updated.\n");
       result = EXIT_SUCCESS;
