@@ -18,18 +18,21 @@
 
 #include <cerrno>
 #include <cstdio>
+#include <cstdlib>
 #include <cstring>
+#include <ctime>
 #include <exception>
 #include <getopt.h>
 #include <iostream>
 #include <memory>
 #include <pthread.h>
 #include <sstream>
-#include <stdlib.h>
 #include <string>
+#ifdef __CYGWIN__
+#include <sys/cygwin.h>
+#endif
 #include <sys/stat.h>
 #include <sys/time.h>
-#include <time.h>
 #include <unistd.h>
 
 #include "Context.h"
@@ -47,7 +50,10 @@
  * not have a short version */
 #define LONG_OPT_ANNOTATE 513
 #define LONG_OPT_NOCACHE 514
-#define LONG_OPT_REQUIRE_MAC 515
+#define LONG_OPT_NODATACACHE 515
+#define LONG_OPT_NOATTRCACHE 516
+#define LONG_OPT_REQUIRE_MAC 517
+#define LONG_OPT_INSECURE 518
 
 using namespace std;
 using namespace encfs;
@@ -85,17 +91,36 @@ struct EncFS_Args {
     ostringstream ss;
     ss << (isDaemon ? "(daemon) " : "(fg) ");
     ss << (isThreaded ? "(threaded) " : "(UP) ");
-    if (idleTimeout > 0) ss << "(timeout " << idleTimeout << ") ";
-    if (opts->checkKey) ss << "(keyCheck) ";
-    if (opts->forceDecode) ss << "(forceDecode) ";
-    if (opts->ownerCreate) ss << "(ownerCreate) ";
-    if (opts->useStdin) ss << "(useStdin) ";
-    if (opts->annotate) ss << "(annotate) ";
-    if (opts->reverseEncryption) ss << "(reverseEncryption) ";
-    if (opts->mountOnDemand) ss << "(mountOnDemand) ";
-    if (opts->delayMount) ss << "(delayMount) ";
-    for (int i = 0; i < fuseArgc; ++i) ss << fuseArgv[i] << ' ';
-
+    if (idleTimeout > 0) {
+      ss << "(timeout " << idleTimeout << ") ";
+    }
+    if (opts->checkKey) {
+      ss << "(keyCheck) ";
+    }
+    if (opts->forceDecode) {
+      ss << "(forceDecode) ";
+    }
+    if (opts->ownerCreate) {
+      ss << "(ownerCreate) ";
+    }
+    if (opts->useStdin) {
+      ss << "(useStdin) ";
+    }
+    if (opts->annotate) {
+      ss << "(annotate) ";
+    }
+    if (opts->reverseEncryption) {
+      ss << "(reverseEncryption) ";
+    }
+    if (opts->mountOnDemand) {
+      ss << "(mountOnDemand) ";
+    }
+    if (opts->delayMount) {
+      ss << "(delayMount) ";
+    }
+    for (int i = 0; i < fuseArgc; ++i) {
+      ss << fuseArgv[i] << ' ';
+    }
     return ss.str();
   }
 
@@ -141,6 +166,12 @@ static void usage(const char *name) {
             "\t\t\t(encfs must be run as root)\n")
        << _("  --reverse\t\t"
             "reverse encryption\n")
+       << _("  --reversewrite\t\t"
+            "reverse encryption with writes enabled\n")
+       << _("  -c, --config=path\t\t"
+            "specifies config file (overrides ENV variable)\n")
+       << _("  -u, --unmount\t\t"
+            "unmounts specified mountPoint\n")
 
        // xgroup(usage)
        << _("  --extpass=program\tUse external program for password prompt\n"
@@ -161,18 +192,21 @@ static void FuseUsage() {
 
   int argc = 2;
   const char *argv[] = {"...", "-h"};
-  fuse_main(argc, const_cast<char **>(argv), (fuse_operations *)NULL, NULL);
+  fuse_main(argc, const_cast<char **>(argv), (fuse_operations *)nullptr,
+            nullptr);
 }
 
 #define PUSHARG(ARG)                        \
   do {                                      \
     rAssert(out->fuseArgc < MaxFuseArgs);   \
     out->fuseArgv[out->fuseArgc++] = (ARG); \
-  } while (0)
+  } while (false)
 
 static string slashTerminate(const string &src) {
   string result = src;
-  if (result[result.length() - 1] != '/') result.append("/");
+  if (result[result.length() - 1] != '/') {
+    result.append("/");
+  }
   return result;
 }
 
@@ -193,6 +227,8 @@ static bool processArgs(int argc, char *argv[],
   out->opts->annotate = false;
   out->opts->reverseEncryption = false;
   out->opts->requireMac = false;
+  out->opts->insecure = false;
+  out->opts->unmount = false;
 
   bool useDefaultFlags = true;
 
@@ -202,37 +238,43 @@ static bool processArgs(int argc, char *argv[],
 
   // leave a space for mount point, as FUSE expects the mount point before
   // any flags
-  out->fuseArgv[1] = NULL;
+  out->fuseArgv[1] = nullptr;
   ++out->fuseArgc;
 
   // TODO: can flags be internationalized?
   static struct option long_options[] = {
-      {"fuse-debug", 0, 0, 'd'},   // Fuse debug mode
-      {"forcedecode", 0, 0, 'D'},  // force decode
+      {"fuse-debug", 0, nullptr, 'd'},   // Fuse debug mode
+      {"forcedecode", 0, nullptr, 'D'},  // force decode
       // {"foreground", 0, 0, 'f'}, // foreground mode (no daemon)
-      {"fuse-help", 0, 0, 'H'},         // fuse_mount usage
-      {"idle", 1, 0, 'i'},              // idle timeout
-      {"anykey", 0, 0, 'k'},            // skip key checks
-      {"no-default-flags", 0, 0, 'N'},  // don't use default fuse flags
-      {"ondemand", 0, 0, 'm'},          // mount on-demand
-      {"delaymount", 0, 0, 'M'},        // delay initial mount until use
-      {"public", 0, 0, 'P'},            // public mode
-      {"extpass", 1, 0, 'p'},           // external password program
+      {"fuse-help", 0, nullptr, 'H'},         // fuse_mount usage
+      {"idle", 1, nullptr, 'i'},              // idle timeout
+      {"anykey", 0, nullptr, 'k'},            // skip key checks
+      {"no-default-flags", 0, nullptr, 'N'},  // don't use default fuse flags
+      {"ondemand", 0, nullptr, 'm'},          // mount on-demand
+      {"delaymount", 0, nullptr, 'M'},        // delay initial mount until use
+      {"public", 0, nullptr, 'P'},            // public mode
+      {"extpass", 1, nullptr, 'p'},           // external password program
       // {"single-thread", 0, 0, 's'},  // single-threaded mode
-      {"stdinpass", 0, 0, 'S'},  // read password from stdin
-      {"syslogtag", 1, 0, 't'},         // syslog tag
-      {"annotate", 0, 0,
-       LONG_OPT_ANNOTATE},                  // Print annotation lines to stderr
-      {"nocache", 0, 0, LONG_OPT_NOCACHE},  // disable caching
-      {"verbose", 0, 0, 'v'},               // verbose mode
-      {"version", 0, 0, 'V'},               // version
-      {"reverse", 0, 0, 'r'},               // reverse encryption
-      {"standard", 0, 0, '1'},              // standard configuration
-      {"paranoia", 0, 0, '2'},              // standard configuration
-      {"require-macs", 0, 0, LONG_OPT_REQUIRE_MAC},  // require MACs
-      {0, 0, 0, 0}};
+      {"stdinpass", 0, nullptr, 'S'},  // read password from stdin
+      {"syslogtag", 1, nullptr, 't'},  // syslog tag
+      {"annotate", 0, nullptr,
+       LONG_OPT_ANNOTATE},  // Print annotation lines to stderr
+      {"nocache", 0, nullptr, LONG_OPT_NOCACHE},         // disable all caching
+      {"nodatacache", 0, nullptr, LONG_OPT_NODATACACHE}, // disable data caching
+      {"noattrcache", 0, nullptr, LONG_OPT_NOATTRCACHE}, // disable attr caching
+      {"verbose", 0, nullptr, 'v'},               // verbose mode
+      {"version", 0, nullptr, 'V'},               // version
+      {"reverse", 0, nullptr, 'r'},               // reverse encryption
+      {"reversewrite", 0, nullptr, 'R'},          // reverse encryption with write enabled
+      {"standard", 0, nullptr, '1'},              // standard configuration
+      {"paranoia", 0, nullptr, '2'},              // standard configuration
+      {"require-macs", 0, nullptr, LONG_OPT_REQUIRE_MAC},  // require MACs
+      {"insecure", 0, nullptr, LONG_OPT_INSECURE},// allows to use null data encryption
+      {"config", 1, nullptr, 'c'},                // command-line-supplied config location
+      {"unmount", 1, nullptr, 'u'},               // unmount
+      {nullptr, 0, nullptr, 0}};
 
-  while (1) {
+  while (true) {
     int option_index = 0;
 
     // 's' : single-threaded mode
@@ -244,10 +286,14 @@ static bool processArgs(int argc, char *argv[],
     // 'S' : password from stdin
     // 'o' : arguments meant for fuse
     // 't' : syslog tag
+    // 'c' : configuration file
+    // 'u' : unmount
     int res =
-        getopt_long(argc, argv, "HsSfvdmi:o:t:", long_options, &option_index);
+        getopt_long(argc, argv, "HsSfvdmi:o:t:c:u", long_options, &option_index);
 
-    if (res == -1) break;
+    if (res == -1) {
+      break;
+    }
 
     switch (res) {
       case '1':
@@ -271,6 +317,19 @@ static bool processArgs(int argc, char *argv[],
       case LONG_OPT_REQUIRE_MAC:
         out->opts->requireMac = true;
         break;
+      case LONG_OPT_INSECURE:
+        out->opts->insecure = true;
+        break;
+      case 'c':
+        /* Take config file path from command 
+         * line instead of ENV variable */
+        out->opts->config.assign(optarg);
+        break;
+      case 'u':
+        //we want to log to console, not to syslog, in case of error
+        out->isDaemon = false;
+        out->opts->unmount = true;
+        break;
       case 'f':
         out->isDaemon = false;
         // this option was added in fuse 2.x
@@ -283,7 +342,7 @@ static bool processArgs(int argc, char *argv[],
         PUSHARG("-d");
         break;
       case 'i':
-        out->idleTimeout = strtol(optarg, (char **)NULL, 10);
+        out->idleTimeout = strtol(optarg, (char **)nullptr, 10);
         out->opts->idleTracking = true;
         break;
       case 'k':
@@ -312,6 +371,12 @@ static bool processArgs(int argc, char *argv[],
          * slowdown. If you are concerned about inconsistencies,
          * please use --nocache. */
         break;
+      case 'R':
+        out->opts->reverseEncryption = true;
+         /* At least this is what the user wants, we will see later
+            if it is possible */
+        out->opts->readOnly = false;
+        break;
       case LONG_OPT_NOCACHE:
         /* Disable EncFS block cache
          * Causes reverse grow tests to fail because short reads
@@ -324,6 +389,21 @@ static bool processArgs(int argc, char *argv[],
         /* Disable kernel dentry cache
          * Fallout unknown, disabling for safety */
         PUSHARG("-oentry_timeout=0");
+#ifdef __CYGWIN__
+        // Should be enforced due to attr_timeout=0, but does not seem to work correctly
+        // https://github.com/billziss-gh/winfsp/issues/155
+        PUSHARG("-oFileInfoTimeout=0");
+#endif
+        break;
+      case LONG_OPT_NODATACACHE:
+        out->opts->noCache = true;
+        break;
+      case LONG_OPT_NOATTRCACHE:
+        PUSHARG("-oattr_timeout=0");
+        PUSHARG("-oentry_timeout=0");
+#ifdef __CYGWIN__
+        PUSHARG("-oFileInfoTimeout=0");
+#endif
         break;
       case 'm':
         out->opts->mountOnDemand = true;
@@ -342,9 +422,9 @@ static bool processArgs(int argc, char *argv[],
         out->opts->passwordProgram.assign(optarg);
         break;
       case 'P':
-        if (geteuid() != 0)
-          RLOG(WARNING) << "option '--public' ignored for non-root user";
-        else {
+        if (geteuid() != 0) {
+          cerr << "option '--public' ignored for non-root user";
+        } else {
           out->opts->ownerCreate = true;
           // add 'allow_other' option
           // add 'default_permissions' option (default)
@@ -355,6 +435,12 @@ static bool processArgs(int argc, char *argv[],
       case 'V':
         // xgroup(usage)
         cerr << autosprintf(_("encfs version %s"), VERSION) << endl;
+#if defined(HAVE_XATTR)
+        // "--verbose" has to be passed before "--version" for this to work.
+        if (out->isVerbose) {
+          cerr << "Compiled with : HAVE_XATTR" << endl;
+        }
+#endif
         exit(EXIT_SUCCESS);
         break;
       case 'H':
@@ -368,12 +454,28 @@ static bool processArgs(int argc, char *argv[],
         // missing parameter for option..
         break;
       default:
-        RLOG(WARNING) << "getopt error: " << res;
+        cerr << "getopt error: " << res;
         break;
     }
   }
 
-  if (!out->isThreaded) PUSHARG("-s");
+  if (!out->isThreaded) {
+    PUSHARG("-s");
+  }
+
+  // for --unmount, we should have exactly 1 argument - the mount point
+  if (out->opts->unmount) {
+    if (optind + 1 == argc) {
+      // unmountPoint is kept as given by the user : in Cygwin, it is used
+      // by pkill to terminate the correct process. We can't then use a
+      // Linux-converted Windows-style mountPoint to unmount...
+      out->opts->unmountPoint = string(argv[optind++]);
+      return true;
+    }
+    // no mount point specified
+    cerr << _("Expecting one argument, aborting.") << endl;
+    return false;
+  }
 
   // we should have at least 2 arguments left over - the source directory and
   // the mount point.
@@ -381,10 +483,11 @@ static bool processArgs(int argc, char *argv[],
     // both rootDir and mountPoint are assumed to be slash terminated in the
     // rest of the code.
     out->opts->rootDir = slashTerminate(argv[optind++]);
-    out->opts->mountPoint = slashTerminate(argv[optind++]);
+    out->opts->unmountPoint = string(argv[optind++]);
+    out->opts->mountPoint = slashTerminate(out->opts->unmountPoint);
   } else {
     // no mount point specified
-    cerr << _("Missing one or more arguments, aborting.");
+    cerr << _("Missing one or more arguments, aborting.") << endl;
     return false;
   }
 
@@ -423,6 +526,13 @@ static bool processArgs(int argc, char *argv[],
 #endif
   }
 
+#ifdef __CYGWIN__
+  // Windows users may use Windows paths
+  // https://cygwin.com/cygwin-api/cygwin-functions.html
+  out->opts->mountPoint = string((char *)cygwin_create_path(CCP_WIN_A_TO_POSIX | CCP_RELATIVE, out->opts->mountPoint.c_str()));
+  out->opts->rootDir = string((char *)cygwin_create_path(CCP_WIN_A_TO_POSIX | CCP_RELATIVE, out->opts->rootDir.c_str()));
+#endif
+
   // sanity check
   if (out->isDaemon && (!isAbsolutePath(out->opts->mountPoint.c_str()) ||
                         !isAbsolutePath(out->opts->rootDir.c_str()))) {
@@ -467,18 +577,38 @@ static bool processArgs(int argc, char *argv[],
   if (!isDirectory(out->opts->rootDir.c_str()) &&
       !userAllowMkdir(out->opts->annotate ? 1 : 0, out->opts->rootDir.c_str(),
                       0700)) {
-    cerr << _("Unable to locate root directory, aborting.");
+    cerr << _("Unable to locate root directory, aborting.") << endl;
     return false;
   }
+#ifdef __CYGWIN__
+  if (isDirectory(out->opts->mountPoint.c_str())) {
+    cerr << _("Mount point must not exist before mouting, aborting.") << endl;
+    return false;
+  }
+  if ((strncmp(out->opts->mountPoint.c_str(), "/cygdrive/", 10) != 0) ||
+      (out->opts->mountPoint.length() != 12)) {
+    cerr << _("A drive is prefered for mouting, ")
+         << _("so a path like X: (or /cygdrive/x) should rather be used. ")
+         << _("Mounting anyway.")  << endl;
+  }
+#else
   if (!isDirectory(out->opts->mountPoint.c_str()) &&
       !userAllowMkdir(out->opts->annotate ? 2 : 0,
                       out->opts->mountPoint.c_str(), 0700)) {
-    cerr << _("Unable to locate mount point, aborting.");
+    cerr << _("Unable to locate mount point, aborting.") << endl;
     return false;
   }
+#endif
 
   // fill in mount path for fuse
   out->fuseArgv[1] = out->opts->mountPoint.c_str();
+#ifdef __CYGWIN__
+  if ((strncmp(out->opts->mountPoint.c_str(), "/cygdrive/", 10) == 0) &&
+      (out->opts->mountPoint.length() == 12)) {
+    out->opts->cygDrive = out->opts->mountPoint.substr(10,1).append(":");
+    out->fuseArgv[1] = out->opts->cygDrive.c_str();
+  }
+#endif
 
   return true;
 }
@@ -486,10 +616,18 @@ static bool processArgs(int argc, char *argv[],
 static void *idleMonitor(void *);
 
 void *encfs_init(fuse_conn_info *conn) {
-  EncFS_Context *ctx = (EncFS_Context *)fuse_get_context()->private_data;
+  auto *ctx = (EncFS_Context *)fuse_get_context()->private_data;
 
   // set fuse connection options
-  conn->async_read = true;
+  conn->async_read = 1u;
+
+#ifdef __CYGWIN__
+  // WinFsp needs this to partially handle read-only FS
+  // See https://github.com/billziss-gh/winfsp/issues/157 for details
+  if (ctx->opts->readOnly) {
+    conn->want |= (conn->capable & FSP_FUSE_CAP_READ_ONLY);
+  }
+#endif
 
   // if an idle timeout is specified, then setup a thread to monitor the
   // filesystem.
@@ -497,11 +635,12 @@ void *encfs_init(fuse_conn_info *conn) {
     VLOG(1) << "starting idle monitoring thread";
     ctx->running = true;
 
-    int res = pthread_create(&ctx->monitorThread, 0, idleMonitor, (void *)ctx);
+    int res =
+        pthread_create(&ctx->monitorThread, nullptr, idleMonitor, (void *)ctx);
     if (res != 0) {
       RLOG(ERROR) << "error starting idle monitor thread, "
                      "res = "
-                  << res << ", errno = " << errno;
+                  << res << ", " << strerror(res);
     }
   }
 
@@ -514,8 +653,6 @@ void *encfs_init(fuse_conn_info *conn) {
   return (void *)ctx;
 }
 
-void encfs_destroy(void *_ctx) {}
-
 int main(int argc, char *argv[]) {
 #if defined(ENABLE_NLS) && defined(LOCALEDIR)
   setlocale(LC_ALL, "");
@@ -526,8 +663,9 @@ int main(int argc, char *argv[]) {
   // anything that comes from the user should be considered tainted until
   // we've processed it and only allowed through what we support.
   std::shared_ptr<EncFS_Args> encfsArgs(new EncFS_Args);
-  for (int i = 0; i < MaxFuseArgs; ++i)
-    encfsArgs->fuseArgv[i] = NULL;  // libfuse expects null args..
+  for (int i = 0; i < MaxFuseArgs; ++i) {
+    encfsArgs->fuseArgv[i] = nullptr;  // libfuse expects null args..
+  }
 
   if (argc == 1 || !processArgs(argc, argv, encfsArgs)) {
     usage(argv[0]);
@@ -535,7 +673,15 @@ int main(int argc, char *argv[]) {
   }
 
   encfs::initLogging(encfsArgs->isVerbose, encfsArgs->isDaemon);
-  ELPP_INITIALIZE_SYSLOG(encfsArgs->syslogTag.c_str(), 0, 0);
+  ELPP_INITIALIZE_SYSLOG(encfsArgs->syslogTag.c_str(), LOG_PID, LOG_USER);
+
+  // Let's unmount if requested
+  if (encfsArgs->opts->unmount) {
+    // We use cout here to avoid logging to stderr (and to mess-up tests output)
+    cout << "Filesystem unmounting: " << encfsArgs->opts->unmountPoint << endl;
+    unmountFS(encfsArgs->opts->unmountPoint.c_str());
+    return 0;
+  }
 
   VLOG(1) << "Root directory: " << encfsArgs->opts->rootDir;
   VLOG(1) << "Fuse arguments: " << encfsArgs->toString();
@@ -578,7 +724,6 @@ int main(int argc, char *argv[]) {
   // encfs_oper.releasedir = encfs_releasedir;
   // encfs_oper.fsyncdir = encfs_fsyncdir;
   encfs_oper.init = encfs_init;
-  encfs_oper.destroy = encfs_destroy;
   // encfs_oper.access = encfs_access;
   encfs_oper.create = encfs_create;
   encfs_oper.ftruncate = encfs_ftruncate;
@@ -591,7 +736,7 @@ int main(int argc, char *argv[]) {
 
   // context is not a smart pointer because it will live for the life of
   // the filesystem.
-  auto ctx = std::shared_ptr<EncFS_Context>(new EncFS_Context);
+  auto ctx = std::make_shared<EncFS_Context>();
   ctx->publicFilesystem = encfsArgs->opts->ownerCreate;
   RootPtr rootInfo = initFS(ctx.get(), encfsArgs->opts);
 
@@ -608,7 +753,7 @@ int main(int argc, char *argv[]) {
     ctx->args = encfsArgs;
     ctx->opts = encfsArgs->opts;
 
-    if (encfsArgs->isThreaded == false && encfsArgs->idleTimeout > 0) {
+    if (!encfsArgs->isThreaded && encfsArgs->idleTimeout > 0) {
       // xgroup(usage)
       cerr << _("Note: requested single-threaded mode, but an idle\n"
                 "timeout was specified.  The filesystem will operate\n"
@@ -630,7 +775,9 @@ int main(int argc, char *argv[]) {
     try {
       time_t startTime, endTime;
 
-      if (encfsArgs->opts->annotate) cerr << "$STATUS$ fuse_main_start" << endl;
+      if (encfsArgs->opts->annotate) {
+        cerr << "$STATUS$ fuse_main_start" << endl;
+      }
 
       // FIXME: workaround for fuse_main returning an error on normal
       // exit.  Only print information if fuse_main returned
@@ -644,9 +791,13 @@ int main(int argc, char *argv[]) {
 
       time(&endTime);
 
-      if (encfsArgs->opts->annotate) cerr << "$STATUS$ fuse_main_end" << endl;
+      if (encfsArgs->opts->annotate) {
+        cerr << "$STATUS$ fuse_main_end" << endl;
+      }
 
-      if (res == 0) returnCode = EXIT_SUCCESS;
+      if (res == 0) {
+        returnCode = EXIT_SUCCESS;
+      }
 
       if (res != 0 && encfsArgs->isDaemon && (oldStderr >= 0) &&
           (endTime - startTime <= 1)) {
@@ -675,7 +826,7 @@ int main(int argc, char *argv[]) {
       pthread_cond_signal(&ctx->wakeupCond);
       pthread_mutex_unlock(&ctx->wakeupMutex);
       VLOG(1) << "joining with idle monitoring thread";
-      pthread_join(ctx->monitorThread, 0);
+      pthread_join(ctx->monitorThread, nullptr);
       VLOG(1) << "join done";
     }
   }
@@ -698,54 +849,29 @@ int main(int argc, char *argv[]) {
     having the filesystem unmounted from underneath open files!
 */
 const int ActivityCheckInterval = 10;
-static bool unmountFS(EncFS_Context *ctx);
 
 static void *idleMonitor(void *_arg) {
-  EncFS_Context *ctx = (EncFS_Context *)_arg;
+  auto *ctx = (EncFS_Context *)_arg;
   std::shared_ptr<EncFS_Args> arg = ctx->args;
 
   const int timeoutCycles = 60 * arg->idleTimeout / ActivityCheckInterval;
-  int idleCycles = -1;
 
   bool unmountres = false;
 
-  // We will notify when FS will be unmounted, so notify that it has just been mounted
-  RLOG(INFO) << "Filesystem mounted: " << arg->opts->mountPoint;
+  // We will notify when FS will be unmounted, so notify that it has just been
+  // mounted
+  RLOG(INFO) << "Filesystem mounted: " << arg->opts->unmountPoint;
 
   pthread_mutex_lock(&ctx->wakeupMutex);
 
   while (ctx->running) {
-    int usage, openCount;
-    ctx->getAndResetUsageCounter(&usage, &openCount);
-
-    if (usage == 0 && ctx->isMounted())
-      ++idleCycles;
-    else {
-      if (idleCycles >= timeoutCycles)
-        RLOG(INFO) << "Filesystem no longer inactive: "
-                   << arg->opts->mountPoint;
-      idleCycles = 0;
+    unmountres = ctx->usageAndUnmount(timeoutCycles);
+    if (unmountres) {
+      break;
     }
 
-    if (idleCycles >= timeoutCycles) {
-      if (openCount == 0) {
-        unmountres = unmountFS(ctx);
-        if (unmountres) {
-          // wait for main thread to wake us up
-          pthread_cond_wait(&ctx->wakeupCond, &ctx->wakeupMutex);
-          break;
-        }
-      } else {
-        RLOG(WARNING) << "Filesystem inactive, but " << openCount
-                      << " files opened: " << arg->opts->mountPoint;
-      }
-    }
-
-    VLOG(1) << "idle cycle count: " << idleCycles << ", timeout after "
-            << timeoutCycles;
-
     struct timeval currentTime;
-    gettimeofday(&currentTime, 0);
+    gettimeofday(&currentTime, nullptr);
     struct timespec wakeupTime;
     wakeupTime.tv_sec = currentTime.tv_sec + ActivityCheckInterval;
     wakeupTime.tv_nsec = currentTime.tv_usec * 1000;
@@ -754,32 +880,13 @@ static void *idleMonitor(void *_arg) {
 
   pthread_mutex_unlock(&ctx->wakeupMutex);
 
-  // If we are here FS has been unmounted, so if we did not unmount ourselves (manual, kill...), notify
-  if (!unmountres)
-    RLOG(INFO) << "Filesystem unmounted: " << arg->opts->mountPoint;
+  // If we are here FS has been unmounted, so if the idleMonitor did not unmount itself,
+  // let's notify (certainly due to a kill signal, a manual unmount...)
+  if (!unmountres) {
+    RLOG(INFO) << "Filesystem unmounted: " << arg->opts->unmountPoint;
+  }
 
   VLOG(1) << "Idle monitoring thread exiting";
 
-  return 0;
-}
-
-static bool unmountFS(EncFS_Context *ctx) {
-  std::shared_ptr<EncFS_Args> arg = ctx->args;
-  if (arg->opts->mountOnDemand) {
-    VLOG(1) << "Detaching filesystem due to inactivity: "
-            << arg->opts->mountPoint;
-
-    ctx->setRoot(std::shared_ptr<DirNode>());
-    return false;
-  } else {
-    // Time to unmount!
-#if FUSE_USE_VERSION < 30
-    fuse_unmount(arg->opts->mountPoint.c_str(), NULL);
-#else
-    fuse_unmount(fuse_get_context()->fuse);
-#endif
-    // fuse_unmount succeeds and returns void
-    RLOG(INFO) << "Filesystem inactive, unmounted: " << arg->opts->mountPoint;
-    return true;
-  }
+  return nullptr;
 }
