@@ -20,9 +20,9 @@
 
 #include "openssl.h"
 
+#include <cstdlib>
 #include <openssl/crypto.h>
 #include <pthread.h>
-#include <stdlib.h>
 
 #define NO_DES
 #include <openssl/rand.h>
@@ -37,20 +37,21 @@ namespace encfs {
 
 unsigned long pthreads_thread_id() { return (unsigned long)pthread_self(); }
 
-static pthread_mutex_t *crypto_locks = NULL;
+static pthread_mutex_t *crypto_locks = nullptr;
 void pthreads_locking_callback(int mode, int n, const char *caller_file,
                                int caller_line) {
   (void)caller_file;
   (void)caller_line;
 
-  if (!crypto_locks) {
+  if (crypto_locks == nullptr) {
     VLOG(1) << "Allocating " << CRYPTO_num_locks() << " locks for OpenSSL";
     crypto_locks = new pthread_mutex_t[CRYPTO_num_locks()];
-    for (int i = 0; i < CRYPTO_num_locks(); ++i)
-      pthread_mutex_init(crypto_locks + i, 0);
+    for (int i = 0; i < CRYPTO_num_locks(); ++i) {
+      pthread_mutex_init(crypto_locks + i, nullptr);
+    }
   }
 
-  if (mode & CRYPTO_LOCK) {
+  if ((mode & CRYPTO_LOCK) != 0) {
     pthread_mutex_lock(crypto_locks + n);
   } else {
     pthread_mutex_unlock(crypto_locks + n);
@@ -58,11 +59,12 @@ void pthreads_locking_callback(int mode, int n, const char *caller_file,
 }
 
 void pthreads_locking_cleanup() {
-  if (crypto_locks) {
-    for (int i = 0; i < CRYPTO_num_locks(); ++i)
+  if (crypto_locks != nullptr) {
+    for (int i = 0; i < CRYPTO_num_locks(); ++i) {
       pthread_mutex_destroy(crypto_locks + i);
+    }
     delete[] crypto_locks;
-    crypto_locks = NULL;
+    crypto_locks = nullptr;
   }
 }
 
@@ -95,7 +97,9 @@ void openssl_shutdown(bool threaded) {
   ENGINE_cleanup();
 #endif
 
-  if (threaded) pthreads_locking_cleanup();
+  if (threaded) {
+    pthreads_locking_cleanup();
+  }
 }
 
 }  // namespace encfs
